class RequestRasterVariable < ActiveRecord::Base

  belongs_to :extract_request
  belongs_to :raster_variable
  belongs_to :raster_dataset
  belongs_to :raster_operation
  belongs_to :sample_geog_level


  def mnemonic
    RequestRasterVariable.mnemonic(raster_variable, raster_operation, raster_dataset, sample_geog_level)
  end

  def self.mnemonic(raster_variable, raster_operation, raster_dataset, sample_geog_level)
    #band = (!raster_dataset.nil? && !raster_dataset.raster_band_index.nil?) ? raster_dataset.raster_band_index : 1
    #band = 1 if band <= 0
    
    geography_name = SampleGeogLevel.geography_name(sample_geog_level)
    
    operation = raster_operation.nil? ? "" : "#{raster_operation.opcode}"
    
    mnem = "#{raster_variable.mnemonic}_#{operation}_#{geography_name}_#{raster_dataset.mnemonic}"
    
    # AreaDataRasterVariableMnemonicLookup(composite_mnemonic: string, mnemonic: string, raster_operation_opcode: string, geog_level: string, dataset_label: string, description: text)
    
    adrvml = AreaDataRasterVariableMnemonicLookup.where(composite_mnemonic: mnem).first
    
    if adrvml.nil?
      
      rd_desc = raster_dataset.long_description
      desc = raster_variable.label + "; " + rd_desc[:years] + "; " + (raster_operation.nil? ? "" : raster_operation.name) + "(" + raster_variable.units + ")"
      
      AreaDataRasterVariableMnemonicLookup.create!(composite_mnemonic: mnem, mnemonic: raster_variable.mnemonic, raster_operation_opcode: operation, geog_level: geography_name, dataset_label: raster_dataset.mnemonic, description: desc)
    end
    
    #"#{raster_variable.mnemonic}_#{operation}_#{band}_#{geography_name}"
    
    mnem
  end
  
  
  ### Moved to SampleGeogLevel
  #def self.geography_name(sample_geog_level)
  #  if sample_geog_level.nil?
  #    ""
  #  else
  #    if sample_geog_level.geolink_variable.nil?
  #      sample_geog_level.terrapop_sample.short_country_name.upcase + "_" + sample_geog_level.country_level.geog_unit.code.upcase
  #    else
  #      sample_geog_level.geolink_variable.mnemonic
  #    end
  #  end
  #end

  def geography_name
    RequestRasterVariable.geography_name(sample_geog_level)
  end


  def mnemonic_sym
    mnemonic.to_sym
  end


  def is_summary?
    raster_operation.nil?
  end


  # raster_operation might be nil if this is a raster-only extract.
  def operation
    raster_operation.nil? ? "" : "#{raster_operation.opcode}"
  end


  # methods needed for syntax file generation.
  # For Variable, they're in the database.
  # For Area_Data_Variable, they're hardcoded similar to the structure here.
  def filename
    raster_variable.filename
  end


  def long_mnemonic
    raster_variable.long_mnemonic
  end


  def raster_categories
    raster_variable.raster_categories
  end


  def data_type
    "decimal"
  end


  # shouldn't matter for CSV extracts.
  def len
    10
  end


  def implied_decimal_places
    0
  end


  def categories
    []
  end


  def label
    mnemonic
  end


  def to_s
    mnemonic
  end

  def self.long_description(request_raster_variables)
    if request_raster_variables.count > 0
      mnemonics_operations = {}
      str = []
      
      request_raster_variables.each do |rrv|
        unless mnemonics_operations.has_key? rrv.raster_variable.mnemonic
          mnemonics_operations[rrv.raster_variable.mnemonic] = []
        end
        mnemonics_operations[rrv.raster_variable.mnemonic] << rrv.raster_operation.name
      end
      
      mnemonics_operations.each do |mnemonic,operations|
        m = mnemonic.dup
        str << m.widthize(16) + operations.join(", ")
      end
      
      str.join("\n")
    else
      "No Raster Operations"
    end
  end

end
