module TerrapopConfigurationModule
  ENVIRONMENTS = "environments"
  APPLICATION  = "application"
  COUNTRIES    = "countries"
  SETTINGS     = "settings"


  def self.before

    unless defined? @@configuration
      str = File.open(Rails.root.to_s + "/config/terrapop.yml", 'r') { |f| f.read }
      str = ERB.new(str).result
      @@configuration = YAML.load(str)
    end

    unless @@configuration.dig(APPLICATION, COUNTRIES).nil?
      if @@configuration[APPLICATION][COUNTRIES][0] == :all
        @@configuration[APPLICATION][COUNTRIES] = Country.order(:short_name).pluck(:short_name)
      end
    end

    if defined? ENV['SERVER_NAME']
      unless ENV['SERVER_NAME'].nil?
        #$stderr.puts "ServerName: #{ENV['SERVER_NAME']}"
        # this area will be arrived upon ONLY in the WebApp part of TP
        # we need to set/update a TerrapopSettings object with our current hostname
        # which is found in ENV['SERVER_NAME']
        begin
          setting = TerrapopSetting.where(name: 'server_name').first
          setting = TerrapopSetting.new if setting.nil?

          # the following line is a workaroud to a known bug in Rails 4.0 4.1
          # they are going to introduce the bugfix in Rails 4.2
          # if you stumble upon this line of code and Rails is 4.2 or more
          # please remove the following line
          setting.data_will_change!
          # end of story, thanks

          setting.data = {'value' => ENV['SERVER_NAME']}
          setting.name = 'server_name'
          setting.save
        rescue

        end
      end
    end

  end


  before

end



class TerrapopConfiguration

  include TerrapopConfigurationModule


  def self.[](key)
    @@configuration.dig(key)
  end

  def self.settings
    @@configuration.dig(APPLICATION, ENVIRONMENTS, Rails.env)
  end


  def self.inspect
    @@configuration.to_s
  end

end
