class AddNewRasterSummarizationV7 < ActiveRecord::Migration
  def change
    sql =<<-SQL
    CREATE OR REPLACE FUNCTION terrapop_glc_binary_summarization_v7(IN sample_geog_lvl_id bigint, IN rasters_id bigint)
      RETURNS TABLE(geog_instance_id bigint, geog_instance_label character varying, binary_area double precision, total_area double precision, percent_area double precision) AS
      $BODY$
            DECLARE

            data_raster text := '';
            area_raster text := '';
            query text := '';

            BEGIN

            SELECT r_table_schema || '.' || table_name as tablename, area_reference_id
            FROM new_rasters nw
            INTO data_raster
            WHERE nw.raster_variable_id = rasters_id;

            RAISE NOTICE '%', data_raster;

            WITH t1 as
            (
            SELECT r_table_schema || '.' || table_name as tablename, area_reference_id
            FROM new_rasters nw
            WHERE nw.raster_variable_id = rasters_id
            )
            SELECT r_table_schema || '.' || table_name as area_reference_table
            INTO area_raster
            from new_rasters nw, t1
            where nw.raster_variable_id = t1.area_reference_id;

            RAISE NOTICE '%', area_raster;

            query := $$ WITH metadata_raster as
            (
            SELECT st_srid(r.rast) as srid
            FROM rasters r
            where r.raster_variable_id = $$ ||rasters_id || $$
            limit 1
            ), bounds as
            (
            SELECT sgl.id as sample_geog_level_id, gi.id as gid, gi.label as label, gi.code as geog_instance_code, st_transform(bound.geog::geometry, md.srid) as geom
            FROM metadata_raster md, sample_geog_levels sgl inner join geog_instances gi on sgl.id = gi.sample_geog_level_id inner join boundaries bound on bound.geog_instance_id = gi.id
            where sgl.id = $$ || sample_geog_lvl_id || $$
            ), data_rast as
            (
            SELECT b.gid as gid, b.label as label, ST_Union(ST_Clip(r.rast, b.geom)) as rast
            FROM bounds b inner join $$ || data_raster || $$ r on ST_Intersects(r.rast, b.geom)
            GROUP BY b.gid, b.label
            ), area_ref_rast as
            (
            SELECT b.gid as gid, b.label as label, ST_Union(ST_Clip(r.rast, b.geom)) as rast
            FROM bounds b inner join $$ || area_raster || $$ r on ST_Intersects(r.rast, b.geom)
            GROUP BY b.gid, b.label
            ),
            binary_area as
            (
            select b.gid, b.label, (ST_SummaryStats(ST_MapAlgebra(b.rast, 1, a.rast, 1, '[rast1]*[rast2]'))).sum as binary_area
            FROM data_rast b inner join area_ref_rast a on b.gid = a.gid
            group by b.gid, b.label, a.rast, b.rast
            ),
            total_area as
            (
            SELECT a.gid as gid, a.label as label, (ST_SummaryStats(a.rast, true)).sum as total_area
            FROM area_ref_rast a group by a.gid, a.label, a.rast order by a.label
            ),
            percent_area as
            (
            select a.gid as gid, a.label as label, a.binary_area/b.total_area as percent_area
            from binary_area a, total_area b where a.gid = b.gid order by a.label
            )
            SELECT b.gid, b.label, b.binary_area, t.total_area, p.percent_area
            FROM binary_area b inner join total_area t on b.gid = t.gid inner join percent_area p on t.gid = p.gid$$;

            RAISE NOTICE  ' % ', query;

            RETURN QUERY execute query;

            END;
            $BODY$
      LANGUAGE plpgsql VOLATILE
      COST 100
      ROWS 1000;    
    SQL
    
    execute(sql)
    
  end
end
